@@ -49,6 +49,10 @@ Return value is undefined.\
   struct channel_data *data = _scm_to_channel_data (channel);
   int res;
   GSSH_VALIDATE_CHANNEL_DATA (data, channel, FUNC_NAME);
+
+  if (! _gssh_channel_parent_session_connected_p (data))
+    guile_ssh_error1 (FUNC_NAME, "Parent session is not connected", channel);
+
   res = ssh_channel_open_session (data->ssh_channel);
   _gssh_log_debug_format(FUNC_NAME, channel, "result: %d", res);
   if (res != SSH_OK)
@@ -78,6 +82,9 @@ Run a shell command CMD without an interactive shell.\
   GSSH_VALIDATE_OPEN_CHANNEL (channel, SCM_ARG1, FUNC_NAME);
   SCM_ASSERT (scm_is_string (cmd), cmd, SCM_ARG2, FUNC_NAME);
 
+  if (! _gssh_channel_parent_session_connected_p (data))
+    guile_ssh_error1 (FUNC_NAME, "Parent session is not connected", channel);
+
   c_cmd = scm_to_locale_string (cmd);
   res = ssh_channel_request_exec (data->ssh_channel, c_cmd);
   _gssh_log_debug_format(FUNC_NAME, scm_list_2 (channel, cmd),
@@ -109,6 +116,10 @@ returned (yet). \
   GSSH_VALIDATE_OPEN_CHANNEL (channel, SCM_ARG1, FUNC_NAME);
 
   cd = _scm_to_channel_data (channel);
+
+  if (! _gssh_channel_parent_session_connected_p (cd))
+    guile_ssh_error1 (FUNC_NAME, "Parent session is not connected", channel);
+
   res = ssh_channel_get_exit_status (cd->ssh_channel);
   _gssh_log_debug_format(FUNC_NAME, channel, "result: %d", res);
   if (res == SSH_ERROR) {
@@ -137,6 +148,9 @@ Return value is undefined.\
   SCM_ASSERT (scm_is_unsigned_integer (exit_status, 0, UINT32_MAX), exit_status,
               SCM_ARG2, FUNC_NAME);
 
+  if (! _gssh_channel_parent_session_connected_p (cd))
+    guile_ssh_error1 (FUNC_NAME, "Parent session is not connected", channel);
+
   res = ssh_channel_request_send_exit_status (cd->ssh_channel,
                                               scm_to_uint32 (exit_status));
   _gssh_log_debug_format(FUNC_NAME, scm_list_2 (channel, exit_status),
@@ -164,6 +178,9 @@ Return value is undefined.\
 
   GSSH_VALIDATE_OPEN_CHANNEL (channel, SCM_ARG1, FUNC_NAME);
 
+  if (! _gssh_channel_parent_session_connected_p (data))
+    guile_ssh_error1 (FUNC_NAME, "Parent session is not connected", channel);
+
   res = ssh_channel_request_pty (data->ssh_channel);
   _gssh_log_debug_format(FUNC_NAME, channel, "result: %d", res);
   if (res != SSH_OK)
@@ -189,6 +206,9 @@ Return value is undefined.\
 
   GSSH_VALIDATE_OPEN_CHANNEL (channel, SCM_ARG1, FUNC_NAME);
 
+  if (! _gssh_channel_parent_session_connected_p (data))
+    guile_ssh_error1 (FUNC_NAME, "Parent session is not connected", channel);
+
   res = ssh_channel_request_shell (data->ssh_channel);
   _gssh_log_debug_format(FUNC_NAME, channel, "result: %d", res);
   if (res != SSH_OK)
@@ -220,6 +240,9 @@ Return value is undefined.\
   SCM_ASSERT (scm_is_string (name), name, SCM_ARG2, FUNC_NAME);
   SCM_ASSERT (scm_is_string (value), value, SCM_ARG3, FUNC_NAME);
 
+  if (! _gssh_channel_parent_session_connected_p (data))
+    guile_ssh_error1 (FUNC_NAME, "Parent session is not connected", channel);
+
   c_name  = scm_to_locale_string (name);
   c_value = scm_to_locale_string (value);
   res = ssh_channel_request_env (data->ssh_channel, c_name, c_value);
@@ -271,6 +294,7 @@ SCM_GSSH_DEFINE (guile_ssh_channel_open_forward,
   struct channel_data *cd = _scm_to_channel_data (channel);
   char *c_remote_host = NULL;
   char *c_source_host = NULL;
+  struct session_data *sd = NULL;
   int res;
 
   SCM_ASSERT (scm_is_string (remote_host), remote_host, SCM_ARG2, FUNC_NAME);
@@ -278,6 +302,19 @@ SCM_GSSH_DEFINE (guile_ssh_channel_open_forward,
   SCM_ASSERT (scm_is_string (source_host), source_host, SCM_ARG4, FUNC_NAME);
   SCM_ASSERT (scm_is_number (local_port),  local_port,  SCM_ARG5, FUNC_NAME);
 
+  if (! cd)
+    guile_ssh_error1 (FUNC_NAME, "Channel is freed: ", channel);
+
+  if (! _gssh_channel_parent_session_connected_p (cd))
+    guile_ssh_error1 (FUNC_NAME, "Parent session is not connected", channel);
+
+  sd = _scm_to_session_data (cd->session);
+  if (! sd)
+    guile_ssh_error1 (FUNC_NAME, "Session is freed: ", cd->session);
+
+  if (! ssh_is_connected (sd->ssh_session))
+    guile_ssh_error1 (FUNC_NAME, "Session is disconnected: ", channel);
+
   scm_dynwind_begin (0);
 
   c_remote_host = scm_to_locale_string (remote_host);
@@ -305,6 +342,9 @@ SCM_GSSH_DEFINE (guile_ssh_channel_open_forward,
 
   scm_dynwind_end ();
 
+  scm_remember_upto_here_1 (channel);
+  scm_remember_upto_here_1 (cd->session);
+
   return _ssh_result_to_symbol (res);
 }
 #undef FUNC_NAME
@@ -419,6 +459,9 @@ eturn value is undefined.\
   SCM_ASSERT (scm_is_unsigned_integer (row, 0, UINT32_MAX), row,
               SCM_ARG2, FUNC_NAME);
 
+  if (! _gssh_channel_parent_session_connected_p (data))
+    guile_ssh_error1 (FUNC_NAME, "Parent session is not connected", channel);
+
   ssh_channel_change_pty_size (data->ssh_channel,
                                scm_to_uint32 (col),
                                scm_to_uint32 (row));
@@ -442,6 +485,9 @@ Return value is undefined.\
   GSSH_VALIDATE_OPEN_CHANNEL (channel, SCM_ARG1, FUNC_NAME);
   SCM_ASSERT (scm_is_symbol (stream_name), stream_name, SCM_ARG2, FUNC_NAME);
 
+  if (! _gssh_channel_parent_session_connected_p (cd))
+    guile_ssh_error1 (FUNC_NAME, "Parent session is not connected", channel);
+
   if (scm_is_eq (stream_name, scm_from_locale_symbol ("stdout")))
     {
       cd->is_stderr = 0;
@@ -474,6 +520,9 @@ Return one of the following symbols: \"stdout\", \"stderr\".\
 
   GSSH_VALIDATE_OPEN_CHANNEL (channel, SCM_ARG1, FUNC_NAME);
 
+  if (! _gssh_channel_parent_session_connected_p (cd))
+    guile_ssh_error1 (FUNC_NAME, "Parent session is not connected", channel);
+
   if (cd->is_stderr == 0)
     return scm_from_locale_symbol ("stdout");
   if (cd->is_stderr == 1)
@@ -524,6 +573,9 @@ SCM_GSSH_DEFINE (gssh_channel_send_eof, "%channel-send-eof",
 
   GSSH_VALIDATE_CHANNEL_DATA (cd, channel, FUNC_NAME);
 
+  if (! _gssh_channel_parent_session_connected_p (cd))
+    guile_ssh_error1 (FUNC_NAME, "Parent session is not connected", channel);
+
   pt_bits = SCM_CELL_TYPE (channel);
   if ((pt_bits & SCM_WRTNG) == 0)
     {
