@@ -71,6 +71,9 @@ static struct symbol_mapping session_options[] = {
   { "stricthostkeycheck", SSH_OPTIONS_STRICTHOSTKEYCHECK },
   { "compression",        SSH_OPTIONS_COMPRESSION        },
   { "compression-level",  SSH_OPTIONS_COMPRESSION_LEVEL  },
+#if HAVE_LIBSSH_0_8_1
+  { "nodelay",            SSH_OPTIONS_NODELAY            },
+#endif
   { "callbacks",          GSSH_OPTIONS_CALLBACKS         },
   { NULL,                 -1 }
 };
@@ -366,6 +369,9 @@ set_option (SCM scm_session, struct session_data* sd, int type, SCM value)
     case SSH_OPTIONS_SSH1:
     case SSH_OPTIONS_SSH2:
     case SSH_OPTIONS_STRICTHOSTKEYCHECK:
+#if HAVE_LIBSSH_0_8_1
+    case SSH_OPTIONS_NODELAY:
+#endif
       return set_bool_opt (session, type, value);
 
     case SSH_OPTIONS_FD:
@@ -473,6 +479,8 @@ Get value of the OPTION.  Throw `guile-ssh-error' on an error.\
   if (res == SSH_ERROR)
     guile_ssh_error1 (FUNC_NAME, "Unable to get value of the option", option);
 
+  scm_remember_upto_here_1 (option);
+
   return value;
 }
 #undef FUNC_NAME
@@ -625,7 +633,11 @@ Return one of the following symbols: 'ok, 'known-changed, 'found-other,\n\
 
   GSSH_VALIDATE_CONNECTED_SESSION (data, session, SCM_ARG1);
 
+#if HAVE_LIBSSH_0_9
+  res = ssh_session_is_known_server (data->ssh_session);
+#else
   res = ssh_is_server_known (data->ssh_session);
+#endif
 
   switch (res)
     {
@@ -672,7 +684,12 @@ Return server's public key.  Throw `guile-ssh-error' on error.\
   kd = (struct key_data *) scm_gc_malloc (sizeof (struct key_data), "ssh key");
   /* TODO: Check `kd' for NULL. */
 
+#if HAVE_LIBSSH_0_8
+  res = ssh_get_server_publickey (sd->ssh_session, &kd->ssh_key);
+#else
   res = ssh_get_publickey (sd->ssh_session, &kd->ssh_key);
+#endif
+
   if (res != SSH_OK)
     guile_ssh_error1 (FUNC_NAME, "Unable to get the server key", session);
 
@@ -695,7 +712,11 @@ Return value is undefined.\
 
   GSSH_VALIDATE_CONNECTED_SESSION (session_data, session, SCM_ARG1);
 
+#if HAVE_LIBSSH_0_9
+  res = ssh_session_update_known_hosts (session_data->ssh_session);
+#else
   res = ssh_write_knownhost (session_data->ssh_session);
+#endif
 
   if (res != SSH_OK)
     guile_ssh_session_error1 (FUNC_NAME, session_data->ssh_session, session);
