@@ -70,7 +70,9 @@
 (test-assert-with-log "connect!, disconnect!"
   (run-client-test
    ;; server
-   simple-server-proc
+   (lambda (server)
+     (let ((s (server-accept server)))
+       (server-handle-key-exchange s)))
    ;; client
    (lambda ()
      (call-with-connected-session
@@ -81,7 +83,9 @@
   2
   (run-client-test
    ;; server
-   simple-server-proc
+   (lambda (server)
+     (let ((s (server-accept server)))
+       (server-handle-key-exchange s)))
    ;; client
    (lambda ()
      (call-with-connected-session
@@ -92,7 +96,9 @@
   'not-known
   (run-client-test
    ;; server
-   simple-server-proc
+   (lambda (server)
+     (let ((s (server-accept server)))
+       (server-handle-key-exchange s)))
    ;; client
    (lambda ()
      (call-with-connected-session
@@ -103,7 +109,9 @@
   'ok
   (run-client-test
    ;; server
-   simple-server-proc
+   (lambda (server)
+     (let ((s (server-accept server)))
+       (server-handle-key-exchange s)))
    ;; client
    (lambda ()
      (let ((res (call-with-connected-session
@@ -117,7 +125,9 @@
   (run-client-test
 
    ;; server
-   simple-server-proc
+   (lambda (server)
+     (let ((s (server-accept server)))
+       (server-handle-key-exchange s)))
 
    ;; client
    (lambda ()
@@ -148,17 +158,17 @@
 
 ;; The procedure called with a wrong object as a parameter which leads to an
 ;; exception.
-(test-error-with-log "userauth-none!, wrong parameter" 'wrong-type-arg
+(test-error-with-log "userauth-none!, session: non-session object" 'wrong-type-arg
   (userauth-none! "Not a session."))
 
 ;; Client tries to authenticate using a non-connected session which leads to
 ;; an exception.
-(test-error-with-log "userauth-none!, not connected" 'wrong-type-arg
+(test-error-with-log "userauth-none!, session: non-connected session" 'wrong-type-arg
   (userauth-none! (make-session-for-test)))
 
 
 ;; Server replies with "success", client receives 'success.
-(test-equal-with-log "userauth-none!, success"
+(test-equal-with-log "userauth-none!, session: connected session"
   'success
   (run-client-test
 
@@ -168,7 +178,7 @@
      (let ((session (server-accept server)))
        (server-handle-key-exchange session)
        (start-session-loop session
-                           (lambda (msg type)
+                           (lambda (msg)
                              (message-auth-set-methods! msg '(none))
                              (message-reply-success msg)))))
 
@@ -191,7 +201,7 @@
      (let ((session (server-accept server)))
        (server-handle-key-exchange session)
        (start-session-loop session
-                           (lambda (msg type)
+                           (lambda (msg)
                              (message-auth-set-methods! msg '(public-key))
                              (message-reply-default msg)))))
 
@@ -214,7 +224,7 @@
      (let ((session (server-accept server)))
        (server-handle-key-exchange session)
        (start-session-loop session
-                           (lambda (msg type)
+                           (lambda (msg)
                              (message-auth-set-methods! msg '(none))
                              (message-reply-success msg 'partial)))))
 
@@ -252,7 +262,7 @@
      (let ((session (server-accept server)))
        (server-handle-key-exchange session)
        (start-session-loop session
-                           (lambda (msg type)
+                           (lambda (msg)
                              (message-auth-set-methods! msg '(password))
                              (message-reply-success msg)))))
 
@@ -273,7 +283,7 @@
      (let ((session (server-accept server)))
        (server-handle-key-exchange session)
        (start-session-loop session
-                           (lambda (msg type)
+                           (lambda (msg)
                              (message-auth-set-methods! msg '(password))
                              (message-reply-success msg)))))
 
@@ -295,7 +305,7 @@
      (let ((session (server-accept server)))
        (server-handle-key-exchange session)
        (start-session-loop session
-                           (lambda (msg type)
+                           (lambda (msg)
                              (message-auth-set-methods! msg '(password))
                              (message-reply-default msg)))))
 
@@ -317,7 +327,7 @@
      (let ((session (server-accept server)))
        (server-handle-key-exchange session)
        (start-session-loop session
-                           (lambda (msg type)
+                           (lambda (msg)
                              (message-auth-set-methods! msg '(password))
                              (message-reply-success msg 'partial)))))
 
@@ -354,7 +364,7 @@
      (let ((session (server-accept server)))
        (server-handle-key-exchange session)
        (start-session-loop session
-                           (lambda (msg type)
+                           (lambda (msg)
                              (message-reply-success msg)))))
    ;; client
    (lambda ()
@@ -373,7 +383,7 @@
      (let ((session (server-accept server)))
        (server-handle-key-exchange session)
        (start-session-loop session
-                           (lambda (msg type)
+                           (lambda (msg)
                              (message-reply-success msg)))))
    ;; client
    (lambda ()
@@ -392,7 +402,7 @@
      (let ((session (server-accept server)))
        (server-handle-key-exchange session)
        (start-session-loop session
-                           (lambda (msg type)
+                           (lambda (msg)
                              (message-reply-success msg)))))
 
    ;; client
@@ -418,6 +428,19 @@
   'wrong-type-arg
   (userauth-public-key/auto! (make-session-for-test)))
 
+
+;;; 'userauth-gssapi!'
+
+;; The procedure called with a wrong object as a parameter which leads to an
+;; exception.
+(test-error-with-log "userauth-gssapi!, wrong parameter" 'wrong-type-arg
+  (userauth-gssapi! "Not a session."))
+
+;; Client tries to authenticate using a non-connected session which leads to
+;; an exception.
+(test-error-with-log "userauth-gssapi!, not connected" 'wrong-type-arg
+  (userauth-gssapi! (make-session-for-test)))
+
 
 ;;;
 
@@ -442,7 +465,7 @@
      (let ((session (server-accept server)))
        (server-handle-key-exchange session)
        (start-session-loop session
-                           (lambda (msg type)
+                           (lambda (msg)
                              (message-auth-set-methods! msg '(password public-key))
                              (message-reply-default msg)))))
 
@@ -459,62 +482,51 @@
 
 ;; make, open, exec
 
-(define (start-server/channel-test server)
-  "Start SERVER for a channel test."
-  (start-server-loop server
-    (let ((channel #f))
-      (lambda (msg)
-        (let ((msg-type (message-get-type msg)))
-          (srvmsg msg-type)
-          (case (car msg-type)
-            ((request-channel-open)
-             (set! channel (message-channel-request-open-reply-accept msg)))
-            ((request-channel)
-             (if (equal? (cadr msg-type) 'channel-request-exec)
-                 (let ((cmd (exec-req:cmd (message-get-req msg))))
-                   (cond
-                    ((string=? cmd "ping")
-                     (write-line "pong" channel)
-                     (message-reply-success msg))
-                    ((string=? cmd "uname") ; For exit status testing
-                     (message-reply-success msg)
-                     (channel-request-send-exit-status channel 0))))
-                 (message-reply-success msg)))
-            (else
-             (message-reply-success msg))))))))
-
 ;; TODO: Fix the bug: the procedure cannot be used to test errors.
 (define (call-with-connected-session/channel-test proc)
+  (define max-tries 30)
   (define (loop count)
     (catch #t
       (lambda ()
         (call-with-connected-session
          (lambda (session)
-           (authenticate-server session)
-           (userauth-none! session)
+           (format-log/scm 'nolog
+                           "call-with-connected-session/channel-test"
+                           "connected in ~d tries: ~a" count session)
+           (let ((result (authenticate-server session)))
+             (format-log/scm 'nolog
+                             "call-with-connected-session/channel-test"
+                             "server authentication result: ~a" result)
+             (when (equal? result 'error)
+               (error "Could not authenticate server" session result)))
+           (let ((result (userauth-none! session)))
+             (format-log/scm 'nolog
+                             "call-with-connected-session/channel-test"
+                             "client authentication result: ~a" result))
+             ;; (unless (equal? result 'ok)
+             ;;   (error "Could not authenticate client" session result)))
            (proc session))))
-      (lambda (key args)
+      (lambda args
         (format-log/scm 'nolog
                         "make-session/channel-test"
-                        "Unable to connect in ~d tries: ~a~%"
-                        (- max-tries count)
-                        session)
+                        "Unable to connect in ~d tries~%"
+                        count)
         (sleep 1)
-        (if (zero? count)
+        (if (= count max-tries)
             (format-log/scm 'nolog
                             "make-session/channel-test"
                             "~a"
                             "Giving up ...")
-            (loop (1- count))))))
-  (loop 30))
+            (loop (1+ count))))))
+  (loop 1))
 
 
-(test-assert "make-channel"
+(test-assert-with-log "make-channel"
   (run-client-test
 
    ;; server
    (lambda (server)
-     (start-server/channel-test server))
+     (start-server/exec server (const #t)))
 
    ;; client
    (lambda ()
@@ -526,7 +538,7 @@
 
    ;; server
    (lambda (server)
-     (start-server/channel-test server))
+     (start-server/exec server (const #t)))
 
    ;; client
    (lambda ()
@@ -540,14 +552,20 @@
 
    ;; server
    (lambda (server)
-     (start-server/channel-test server))
+     (start-server/exec server (const #t)))
 
    ;; client
    (lambda ()
      (call-with-connected-session/channel-test
       (lambda (session)
+        (format-log/scm 'nolog "channel-open-session [client]"
+                        "session: ~a" session)
         (let ((channel (make-channel session)))
+          (format-log/scm 'nolog "channel-open-session [client]"
+                          "channel: ~a" channel)
           (channel-open-session channel)
+          (format-log/scm 'nolog "channel-open-session [client]"
+                          "channel 2: ~a" channel)
           (not (port-closed? channel))))))))
 
 ;; Client sends "ping" as a command to execute, server replies with "pong"
@@ -556,7 +574,7 @@
 
    ;; server
    (lambda (server)
-     (start-server/channel-test server))
+     (start-server/exec server (const #t)))
 
    ;; client
    (lambda ()
@@ -576,7 +594,7 @@
 
    ;; server
    (lambda (server)
-     (start-server/channel-test server))
+     (start-server/exec server (const #t)))
 
    ;; client
    (lambda ()
@@ -584,7 +602,7 @@
       (lambda (session)
         (let ((channel (make-channel session)))
           (channel-open-session channel)
-          (channel-request-exec channel "uname")
+          (channel-request-exec channel "exit status")
           (channel-get-exit-status channel)))))))
 
 (test-assert-with-log "channel-request-exec, printing a freed channel"
@@ -592,23 +610,33 @@
 
    ;; server
    (lambda (server)
-     (start-server/channel-test server))
+     (start-server/exec server (const #t)))
 
    ;; client
    (lambda ()
      (call-with-connected-session/channel-test
       (lambda (session)
         (let ((channel (make-channel session)))
+          (format-log/scm 'nolog "channel-request-exec, printing a freed channel"
+                          "channel 0: ~a" channel)
           (channel-open-session channel)
-          (channel-request-exec channel "uname")
+          (format-log/scm 'nolog "channel-request-exec, printing a freed channel"
+                          "channel 1: ~a" channel)
+          (channel-request-exec channel "exit status")
+          (format-log/scm 'nolog "channel-request-exec, printing a freed channel"
+                          "channel 2: ~a" channel)
           (close channel)
+          (format-log/scm 'nolog "channel-request-exec, printing a freed channel"
+                          "channel: ~a" channel)
           (string-match "#<unknown channel \\(freed\\) [0-9a-f]+>"
                         (object->string channel))))))))
 
 (test-error-with-log "channel-get-exit-status, freed channel"
   'wrong-type-arg
   (run-client-test
-   start-server/channel-test
+   ;; server
+   (lambda (server)
+     (start-server/exec server (const #t)))
    ;; client
    (lambda ()
      (call-with-connected-session
@@ -617,7 +645,7 @@
         (userauth-none! session)
         (let ((channel (make-channel session)))
           (channel-open-session channel)
-          (channel-request-exec channel "uname")
+          (channel-request-exec channel "exit status")
           (close channel)
           (channel-get-exit-status channel)))))))
 
@@ -656,29 +684,35 @@
 
 
 (test-assert-with-log "data transferring, bytevector"
-  (let ((vect-size 10)
-        (vect-fill 10))
+  (run-client-test
 
-    (run-client-test
+   ;; server
+   (lambda (server)
+     (use-modules (rnrs bytevectors)
+                  (rnrs io ports))
+     (start-server/dt-test server
+                           (lambda (channel)
+                             (let ((v (get-bytevector-n channel 10)))
+                               (put-bytevector channel v)))))
 
-     ;; server
-     (lambda (server)
-       (start-server/dt-test server
-                             (lambda (channel)
-                               (let ((v (get-bytevector-n channel vect-size)))
-                                 (put-bytevector channel v)))))
-
-     ;; client
-     (lambda ()
-       (call-with-connected-session/channel-test
-        (lambda (session)
-          (let ((channel (make-channel/dt-test session))
-                (vect    (make-bytevector vect-size vect-fill)))
-            (put-bytevector channel vect)
-            (poll channel
-                  (lambda args
-                    (let ((res (get-bytevector-n channel vect-size)))
-                      (equal? res vect)))))))))))
+   ;; client
+   (lambda ()
+     (call-with-connected-session/channel-test
+      (lambda (session)
+        (let* ((vect-size 10)
+               (channel   (make-channel/dt-test session))
+               (vect      (make-bytevector vect-size 42)))
+          (format-log/scm 'nolog
+                          "data transferring, bytevector"
+                          "vect: ~a" vect)
+          (put-bytevector channel vect)
+          (poll channel
+                (lambda args
+                  (let ((res (get-bytevector-n channel vect-size)))
+                    (format-log/scm 'nolog
+                                    "data transferring, bytevector"
+                                    "res: ~a" res)
+                    (equal? res vect))))))))))
 
 
 ;;;
