@@ -22,9 +22,13 @@
 (use-modules (srfi srfi-64)
              (ssh session)
              (ssh log)
+             (ssh version)
              ;; Helper procedures
              (tests common))
 
+(define %libssh-minor-version
+  (string->number (cadr (string-split (get-libssh-version) #\.))))
+
 (set-log-verbosity! 'functions)
 
 (test-begin-with-log "session")
@@ -68,11 +72,15 @@
                    (ssh2         #f #t)
                    (log-verbosity nolog rare protocol packet functions
                                   nolog)
-                   (compression   "yes" "no")
                    (compression-level 1 2 3 4 5 6 7 8 9)
+                   (compression   "yes" "no")
                    (callbacks     ((user-data . "hello")
                                    (global-request-callback . ,(const #f))))))
         (res #t))
+
+    (if (>= %libssh-minor-version 8)
+        (set! options (cons  '(nodelay #f #t) options)))
+
     (for-each
      (lambda (opt)
        (for-each
@@ -98,6 +106,10 @@
                    (callbacks         "not a list"
                                       ((global-request-callback . #f)))))
         (res #t))
+
+    (if (>= %libssh-minor-version 8)
+        (set! options (cons '(nodelay 12345 "string") options)))
+
     (for-each
      (lambda (opt)
        (for-each
